@@ -10,7 +10,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Id: pgtcl.c,v 1.3 2004/06/13 20:44:59 lbayuk Exp $
+ *	  $Id: pgtcl.c,v 1.4 2004/11/12 02:47:11 lbayuk Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -237,6 +237,18 @@ Pgtcl_Init(Tcl_Interp *interp)
 						  (ClientData) NULL, 
 						  (Tcl_CmdDeleteProc *) NULL);
 
+	Tcl_CreateObjCommand(interp,
+						  "pg_escape_bytea",
+						  Pg_escape_bytea,
+						  (ClientData) NULL, 
+						  (Tcl_CmdDeleteProc *) NULL);
+
+	Tcl_CreateObjCommand(interp,
+						  "pg_unescape_bytea",
+						  Pg_unescape_bytea,
+						  (ClientData) NULL, 
+						  (Tcl_CmdDeleteProc *) NULL);
+
 	Tcl_CreateObjCommand(interp,
 						  "pg_transaction_status",
 						  Pg_transaction_status,
