@@ -8,7 +8,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Id: pgtclCmds.c,v 1.5 2004/06/13 20:44:59 lbayuk Exp $
+ *	  $Id: pgtclCmds.c,v 1.6 2004/11/12 02:47:31 lbayuk Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -830,10 +830,15 @@ Pg_exec_params(ClientData cData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST ob
 
 /**********************************
  * pg_result_errorfield_code
- Translate error fieldName to fieldCode for pg_result -errorField.
+ Translate error fieldName to fieldCode for pg_result -error?Field?
  Valid fieldNames are strings matching the constant name without PG_DIAG_,
 such as "SEVERITY", or the single letter code which is the value of the
 constant, like 'S'. See postgres_ext.h for the list.
+ For compatibility with Gborg pgtcl-1.4, these changes have been made:
+    Accept field names without MESSAGE_, SOURCE_, STATEMENT_ prefix.
+    Do case insensitive compares (Gborg pgtcl uses lower case names only,
+and previous versions of this code used upper case only.)
+
  Returns a valid PG_DIAG_* constant, or 0 if there is no match.
  **********************************/
 static int
@@ -852,20 +857,35 @@ pg_result_errorfield_code(char *fieldName)
 		{ "SOURCE_FILE",		PG_DIAG_SOURCE_FILE },
 		{ "SOURCE_LINE",		PG_DIAG_SOURCE_LINE },
 		{ "SOURCE_FUNCTION",	PG_DIAG_SOURCE_FUNCTION },
+        /* These are for compatibility with Gborg pgtcl-1.4: */
+		{ "PRIMARY",            PG_DIAG_MESSAGE_PRIMARY },
+		{ "DETAIL",             PG_DIAG_MESSAGE_DETAIL },
+		{ "HINT",               PG_DIAG_MESSAGE_HINT },
+		{ "POSITION",           PG_DIAG_STATEMENT_POSITION },
+		{ "FILE",               PG_DIAG_SOURCE_FILE },
+		{ "LINE",               PG_DIAG_SOURCE_LINE },
+		{ "FUNCTION",           PG_DIAG_SOURCE_FUNCTION },
 		{ 0, '\0'}};
 
 	struct errorfield_names_t *ep = errorfield_names;
+	char field1;
 
 	if (!fieldName || !fieldName[0])
 		return 0;
 	if (fieldName[1])
-		/* Check for exact word match if length>1 */
-		while (ep->fieldName && strcmp(ep->fieldName, fieldName) != 0)
+	{
+		/* Check for exact word match if length>1, case insensitively */
+		while (ep->fieldName &&
+				!Tcl_StringCaseMatch(fieldName, ep->fieldName, 1))
 		 	ep++;
-	else
-		/* Check for single-character code match. */
-		while (ep->fieldCode && ep->fieldCode != fieldName[0])
-			 ep++;
+	} else {
+		/* Check for single-character code match, case insensitive. Note
+			these are being checked against the PG_DIAG_* values, which
+			are defined in postgres_ext.h as single upper-case chars.
+		*/
+		field1 = toupper(fieldName[0]);
+		while (ep->fieldCode && ep->fieldCode != field1) ep++;
+	}
 	return ep->fieldCode;
 }
 
@@ -882,11 +902,16 @@ pg_result_errorfield_code(char *fieldName)
 
 	-status the status of the result
 
-	-error	the error message, if the status indicates error; otherwise
-		an empty string
-
-	-errorField c
-		the error message subfield (e.g. SEVERITY)
+	-error	?code?
+	-errorField ?code?
+		If the status does not indicate an error, returns an empty string.
+		Else, if no code is provided, returns the current error message.
+		Else, the code names an error message subfield or abbreviation,
+		and the value of that error field is returned if valid and available.
+		Else, an empty string is returned.
+		Note: -errorField was introduced first, but for compatibility with
+		Gborg pgtcl 1.4 the error field code was made optional to -error
+		and now we implement both of these as the same.
 
 	-conn	the connection that produced the result
 
@@ -1014,17 +1039,7 @@ Pg_result(ClientData cData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[])
 			}
 
 		case OPT_ERROR:
-			{
-				if (objc != 3)
-				{
-					Tcl_WrongNumArgs(interp, 3, objv, NULL);
-					return TCL_ERROR;
-				}
-
-				Tcl_SetObjResult(interp,
-					 Tcl_NewStringObj(PQresultErrorMessage(result), -1));
-				return TCL_OK;
-			}
+			/* Fall through - these subcommands are now identical */
 
 		case OPT_ERRORFIELD:
 			{
@@ -1032,9 +1047,16 @@ Pg_result(ClientData cData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[])
 				int		fieldCode;
 				char   *errorField;
 
+				if (objc == 3)
+				{
+					Tcl_SetObjResult(interp,
+						Tcl_NewStringObj(PQresultErrorMessage(result), -1));
+					return TCL_OK;
+				}
+
 				if (objc != 4)
 				{
-					Tcl_WrongNumArgs(interp, 3, objv, "fieldName");
+					Tcl_WrongNumArgs(interp, 3, objv, "?fieldName?");
 					return TCL_ERROR;
 				}
 
@@ -3309,6 +3331,120 @@ Pg_quote(ClientData cData, Tcl_Interp *interp, int objc,
 	return TCL_OK;
 }
 
+/***********************************
+ * Pg_escape_bytea
+   Escape a binary string for inclusion in SQL queries as a bytea type.
+   See libpq PQescapeBytea.
+   In summary, this produces octal escapes as \\nnn (with 2 literal
+   backslashes before) and \' for single quote and \\\\ (4 backslashes)
+   for backslash. The doubling of backslashes is due to PostgreSQL
+   parsing the data once for SQL syntax, and again for bytea input.
+   Note: This function is NOT the inverse of Pg_unescape_bytea (cf).
+
+   Syntax:
+   pg_escape_bytea binary_string
+
+***********************************/
+int
+Pg_escape_bytea(ClientData cData, Tcl_Interp *interp, int objc,
+				 Tcl_Obj *CONST objv[])
+{
+	unsigned char	   *from_binary;
+	size_t         		from_len;
+	unsigned char	   *to_string;
+	size_t 				to_len;
+
+	if (objc != 2)
+	{
+		Tcl_WrongNumArgs(interp, 1, objv, "binaryString");
+		return TCL_ERROR;
+	}
+
+	/*
+	 * Get the source data as a Tcl ByteArray object
+	 */
+	from_binary = Tcl_GetByteArrayFromObj(objv[1], &from_len);
+
+	/*
+	 * Escape the data. libpq allocates the memory for us.
+	 * Note to_len includes the terminating null byte.
+	 * (The libpq documentation doesn't say, but this can return NULL.)
+	 */
+	to_string = PQescapeBytea(from_binary, from_len, &to_len);
+	if (!to_string)
+	{
+		Tcl_AppendResult(interp, "pg_escape_bytea: failed to get memory\n", 0);
+		return TCL_ERROR;
+	}
+
+	/*
+	 * Copy the result to the interpreter as a string object.
+	 */
+	Tcl_SetObjResult(interp, Tcl_NewStringObj(to_string, to_len-1));
+
+	/*
+	 * Free libpq-allocated memory
+	 */
+	PQfreemem(to_string);
+
+	return TCL_OK;
+}
+
+/***********************************
+ * Pg_unescape_bytea
+   Unescape a string from a PostgreSQL bytea data type and return the
+   original binary data as a Tcl binary object.
+   See libpq PQunescapeBytea.
+   In summary, this takes \nnn octal escapes and produces the byte
+   equivalent to nnn, and any other \c becomes c.
+
+   Note: This function is NOT the inverse of Pg_escape_bytea. That
+   function produces doubled backslashes, and this function expects
+   single backslashes. That's because pg_escape_bytea is meant to
+   escape binary data for quoted SQL strings in SELECT,  INSERT, etc.
+   which goes through two levels of parsing. pg_unescape_bytea is
+   used to retrieve binary data returned by a query on a bytea column,
+   which has only had one level of escaping performed on it.
+
+   Syntax:
+   pg_unescape_bytea string
+
+***********************************/
+int
+Pg_unescape_bytea(ClientData cData, Tcl_Interp *interp, int objc,
+				 Tcl_Obj *CONST objv[])
+{
+	unsigned char	   *to_binary;
+	size_t 				to_len;
+
+	if (objc != 2)
+	{
+		Tcl_WrongNumArgs(interp, 1, objv, "string");
+		return TCL_ERROR;
+	}
+
+	/*
+	 * Unescape the data. libpq allocates the memory for us.
+	 */
+	to_binary = PQunescapeBytea(Tcl_GetString(objv[1]), &to_len);
+	if (!to_binary)
+	{
+		Tcl_AppendResult(interp, "pg_unescape_bytea: failed to get memory\n", 0);
+		return TCL_ERROR;
+	}
+
+	/*
+	 * Copy the result to the interpreter as a ByteArray (binary) object.
+	 */
+	Tcl_SetObjResult(interp, Tcl_NewByteArrayObj(to_binary, to_len));
+
+	/*
+	 * Free libpq-allocated memory
+	 */
+	PQfreemem(to_binary);
+
+	return TCL_OK;
+}
 
 /**********************************
  * pg_transaction_status
