@@ -6,7 +6,7 @@
  * Portions Copyright (c) 1996-2004, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
- * $Id: pgtclCmds.h,v 1.3 2004/06/13 20:44:59 lbayuk Exp $
+ * $Id: pgtclCmds.h,v 1.4 2004/11/12 02:47:11 lbayuk Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -172,6 +172,12 @@ extern int Pg_escape_string(
 extern int Pg_quote(
   ClientData cData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
 
+extern int Pg_escape_bytea(
+  ClientData cData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
+
+extern int Pg_unescape_bytea(
+  ClientData cData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
+
 extern int Pg_transaction_status(
   ClientData cdata, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]);
 
